package app

import (
	"context"
	"errors"
	"fmt"
	"gitea.pena/PenaSide/common/closer"
	"gitea.pena/PenaSide/common/mongo"
	"gitea.pena/PenaSide/treasurer/internal/server/grpc"
	"gitea.pena/PenaSide/treasurer/internal/server/http"
	"time"

	"gitea.pena/PenaSide/treasurer/internal/initialize"
	"gitea.pena/PenaSide/treasurer/internal/worker"
	"go.uber.org/zap"
)

func Run(ctx context.Context, config initialize.Config, logger *zap.Logger) (appErr error) {
	defer func() {
		if recovered := recover(); recovered != nil {
			appErr = errors.New("recovered panic on application run")
			logger.Error("recovered panic on application run", zap.Any("recovered", recovered))
		}
	}()

	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	shutdownGroup := closer.NewCloserGroup()

	mongoDB, err := mongo.Connect(ctx, &mongo.ConnectDeps{
		Configuration: &config.Database,
		Timeout:       10 * time.Second,
	})
	if err != nil {
		return fmt.Errorf("failed connection to db: %w", err)
	}

	repositories, err := initialize.NewRepositories(initialize.RepositoriesDeps{
		Logger:   logger,
		Database: mongoDB,
	})
	if err != nil {
		return err
	}

	clients, err := initialize.NewClients(initialize.ClientsDeps{
		Logger: logger,
		Config: config,
	})
	if err != nil {
		return err
	}

	services, err := initialize.NewServices(initialize.ServicesDeps{
		Logger:       logger,
		Repositories: *repositories,
		Clients:      *clients,
		Config:       config,
	})
	if err != nil {
		return err
	}

	workers, err := initialize.NewWorkers(initialize.WorkersDeps{
		Logger:  logger,
		Service: services.Mock,
	})

	if err != nil {
		return err
	}

	controllers, err := initialize.NewControllers(initialize.ControllersDeps{
		Logger:       logger,
		Services:     *services,
		Repositories: *repositories,
	})
	if err != nil {
		return err
	}

	httpServer := http.NewServer(http.ServerConfig{
		Logger:      logger,
		Controllers: []http.Controller{controllers.YandexStatusREST, controllers.PaymentRest},
	})

	grpcServer, err := grpc.NewGRPC(logger)
	if err != nil {
		return err
	}

	grpcServer.Register(controllers.PaymentGRPC)

	go func() {
		if err := httpServer.Start(config.HttpURL); err != nil {
			logger.Error("Client server startup error", zap.Error(err))
			cancel()
		}
	}()

	go grpcServer.Run(config.GrpcURL)
	go worker.Run(ctx, workers)

	httpServer.ListRoutes()

	shutdownGroup.Add(closer.CloserFunc(mongoDB.Client().Disconnect))
	shutdownGroup.Add(closer.CloserFunc(httpServer.Shutdown))
	shutdownGroup.Add(closer.CloserFunc(grpcServer.Stop))

	<-ctx.Done()

	timeoutCtx, timeoutCancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer timeoutCancel()
	if err := shutdownGroup.Call(timeoutCtx); err != nil {
		if errors.Is(err, context.DeadlineExceeded) {
			logger.Error("Shutdown timed out", zap.Error(err))
		} else {
			logger.Error("Failed to shutdown services gracefully", zap.Error(err))
		}
		return err
	}

	logger.Info("Application has stopped")
	return nil
}
