package grpc

import (
	"context"
	"fmt"
	"net"
	"time"

	"gitea.pena/PenaSide/treasurer/internal/proto/treasurer"
	grpc_middleware "github.com/grpc-ecosystem/go-grpc-middleware"
	grpc_zap "github.com/grpc-ecosystem/go-grpc-middleware/logging/zap"
	grpc_recovery "github.com/grpc-ecosystem/go-grpc-middleware/recovery"
	"go.uber.org/zap"
	"google.golang.org/grpc"
)

type GRPC struct {
	grpc   *grpc.Server
	logger *zap.Logger
}

func NewGRPC(logger *zap.Logger) (*GRPC, error) {
	grpcStreamInterceptor := grpc.StreamInterceptor(grpc_middleware.ChainStreamServer(
		grpc_zap.StreamServerInterceptor(logger),
		grpc_recovery.StreamServerInterceptor(),
	))

	grpcUnaryInterceptor := grpc.UnaryInterceptor(grpc_middleware.ChainUnaryServer(
		grpc_zap.UnaryServerInterceptor(logger),
		grpc_recovery.UnaryServerInterceptor(),
	))

	return &GRPC{
		grpc:   grpc.NewServer(grpcStreamInterceptor, grpcUnaryInterceptor, grpc.ConnectionTimeout(5*time.Second)),
		logger: logger,
	}, nil
}

func (s *GRPC) Run(addr string) {
	connectionString := fmt.Sprintf(addr)

	s.logger.Info("Starting GRPC Server", zap.String("host", connectionString))

	if err := s.listen(connectionString); err != nil && err != grpc.ErrServerStopped {
		s.logger.Error("GRPC Listen error", zap.Error(err))
	}
}

func (s *GRPC) Stop(_ context.Context) error {
	s.grpc.GracefulStop()
	s.logger.Info("Shutting down GRPC server...")

	return nil
}

func (s *GRPC) Register(server treasurer.TreasurerServiceServer) *GRPC {
	treasurer.RegisterTreasurerServiceServer(s.grpc, server)

	return s
}

func (s *GRPC) listen(address string) error {
	listener, err := net.Listen("tcp", address)
	if err != nil {
		return err
	}

	return s.grpc.Serve(listener)
}
